<?php

namespace FiloBlu\Refilo\Remote\Entity;

use FiloBlu\Refilo\Remote\Entity\Provider\PriceProvider;
use FiloBlu\Refilo\Remote\Entity\Provider\ProductAttributeProvider;
use FiloBlu\Refilo\Remote\Entity\Provider\StockProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status as AttributeStatus;
use Magento\Catalog\Model\Product\Visibility;
use Magento\ConfigurableProduct\Api\Data\OptionInterface;

/**
 * Class MagentoToRefiloProductAdapter
 * @package FiloBlu\Refilo\Remote\Entity
 */
class MagentoToRefiloProductAdapter
{
    /**
     * @var array
     */
    protected $prices = [];

    /**
     * @var array
     */
    protected $stocks = [];
    /**
     * @var ProductAttributeProvider
     */
    private $productAttributeProvider;
    /**
     * @var StockProvider
     */
    private $stockProvider;
    /**
     * @var PriceProvider
     */
    private $priceProvider;

    /**
     * MagentoToRefiloProductAdapter constructor.
     * @param ProductAttributeProvider $productAttributeProvider
     * @param StockProvider $stockProvider
     * @param PriceProvider $priceProvider
     */
    public function __construct(
        ProductAttributeProvider $productAttributeProvider,
        StockProvider $stockProvider,
        PriceProvider $priceProvider
    ) {
        $this->productAttributeProvider = $productAttributeProvider;
        $this->stockProvider = $stockProvider;
        $this->priceProvider = $priceProvider;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function adapt(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        $docType = Product::DOCTYPE_SPEC_DISABLED;
        $visibility = '0';

        if ((int)$magentoProduct->getStatus() === AttributeStatus::STATUS_ENABLED) {
            $docType = Product::DOCTYPE_SPEC_ENABLED;
            $visibility = ((int)$magentoProduct->getVisibility() === Visibility::VISIBILITY_NOT_VISIBLE ? '0' : '1');
        }

        $data = [
            Product::DOCTYPE          => $docType,
            Product::ID               => (int)$magentoProduct->getEntityId(),
            Product::SKU              => $magentoProduct->getSku(),
            Product::TYPE             => $magentoProduct->getTypeId(),
            Product::NAME             => $magentoProduct->getName(),
            Product::CREATED_AT       => $magentoProduct->getCreatedAt(),
            Product::UPDATED_AT       => $magentoProduct->getUpdatedAt(),
            Product::WEIGHT           => $magentoProduct->getWeight(),
            Product::META_TITLE       => $magentoProduct->getMetaTitle(),
            Product::META_KEYWORDS    => $magentoProduct->getMetaKeywords(),
            Product::META_DESCRIPTION => $magentoProduct->getMetaDescription(),
            Product::VISIBLE          => $visibility,
            //Product::CATEGORIES       => array_map('intval', $magentoProduct->getCategoryIds()),
            Product::ATTRIBUTES       => $this->productAttributeProvider->getAttributesValues($magentoProduct),
            Product::FILTERS          => $this->productAttributeProvider->getFilterableAttributes($magentoProduct),
            Product::SORTABLES        => $this->productAttributeProvider->getSortableAttributes($magentoProduct),
            Product::UPSELL           => $magentoProduct->getUpSellProductIds(),
            Product::CROSSSELL        => $magentoProduct->getCrossSellProductIds(),
            Product::RELATED          => $magentoProduct->getRelatedProductIds(),
            Product::ASSETS           => $this->getAssets($magentoProduct),
            Product::INVENTORY        => $this->getStockFor($magentoProduct)
        ];

        $prices = $this->getPricesFor($magentoProduct);
        $refiloProduct->setData(array_merge($data, $prices));
        $this->fillChildren($magentoProduct, $refiloProduct);
        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @return array
     */
    protected function getAssets(ProductInterface $magentoProduct): array
    {
        $assets = [];
        $gallery = $magentoProduct->getData('media_gallery');

        foreach ($gallery['images'] as $image) {
            // Skip image is hide
            if (isset($image["disabled"]) && $image["disabled"] == 1) {
                continue;
            }
            $assets['images'][] = [
                'label' => $image['label'] ?? null, // TODO : remove?
                'alt'   => $image['label'] ?? null,
                'file'  => ($image['file']) ? "/catalog/product{$image['file']}" : null
            ];
        }

        $assets['placeholders'] = $this->productAttributeProvider->getImagePlaceholdersAttributes($magentoProduct);

        return $assets;
    }

    /**
     * @param ProductInterface $product
     * @return mixed
     */
    public function getStockFor(ProductInterface $product)
    {
        $this->init($product->getStore());

        if (isset($this->stocks[$product->getStoreId()][$product->getEntityId()])) {
            return $this->stocks[$product->getStoreId()][$product->getEntityId()];
        }
        return [];
    }

    /**
     * @param $store
     */
    public function init($store)
    {
        $storeId = $store->getId();

        if (!isset($this->stocks[$storeId])) {
            $provider = $this->stockProvider->forStore($store);
            foreach ($provider as $item) {
                $this->stocks[$storeId][$item->getId()] = $item->getData(Product::INVENTORY);
            }
        }
        if (!isset($this->prices[$storeId])) {
            $provider = $this->priceProvider->forStore($store);
            foreach ($provider as $item) {
                $this->prices[$storeId][$item->getId()] = $item->getData();
            }
        }
    }

    /**
     * @param ProductInterface $product
     * @return mixed
     */
    public function getPricesFor(ProductInterface $product)
    {
        $this->init($product->getStore());

        if (isset($this->prices[$product->getStoreId()][$product->getEntityId()])) {
            return $this->prices[$product->getStoreId()][$product->getEntityId()];
        }
        return [];
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    protected function fillChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        switch ($magentoProduct->getTypeId()) {
            case 'configurable':
                return $this->getConfigurableChildren($magentoProduct, $refiloProduct);
            case 'bundle':
                return $this->getBundleChildren($magentoProduct, $refiloProduct);
            case 'grouped':
                return $this->getGroupedChildren($magentoProduct, $refiloProduct);
        }

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getConfigurableChildren(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        if ($magentoProduct->getTypeId() === 'configurable') {
            $storeId = $magentoProduct->getStore()->getId();
            $extensionAttributes = $magentoProduct->getExtensionAttributes();

            $refiloProduct->setChildren(array_map('intval', array_values($extensionAttributes->getConfigurableProductLinks())));
            $productOptions = $extensionAttributes->getConfigurableProductOptions() ?? [];

            $variants = [];
            foreach ($productOptions as $option) {
                $attributeCode = $option->getProductAttribute()->getAttributeCode();
                $variant = [
                    'id'             => $option->getId(),
                    'type'           => $option->getProductAttribute()->getData('frontend_input'),
                    'position'       => $option->getPosition(),
                    'label'          => $option->getProductAttribute()->getStoreLabel($storeId),
                    'use_default'    => (int)$option->getIsUseDefault(),
                    'attribute_id'   => $option->getAttributeId(),
                    'attribute_code' => $attributeCode,
                    'values'         => $this->getOptions($option),
                    'options'        => []
                ];

                $additionalData = json_decode($option->getProductAttribute()->getData('additional_data'), true);

                if ($additionalData) {
                    $variant['rendered'] = isset($additionalData['swatch_input_type']) ? $additionalData['swatch_input_type'] : '';
                }

                $options = [];
                foreach ($variant['values'] as $value) {
                    $options[] = (int)$value['value_index'];
                }

                $variant['options'] = $options;

                $variants[] = $variant;
            }

            $salableChildren = $magentoProduct->getTypeInstance()->getSalableUsedProducts($magentoProduct, null);

            $filters = [];
            foreach ($salableChildren as $salableChild) {
                $filterable = $this->productAttributeProvider->getFilterableAttributes($salableChild);

                foreach ($filterable as $attributeCode => $value) {
                    // TODO: handle cases wich not are multiselect or select
                    if (!isset($value['key']) || !$value['key']) {
                        continue;
                    }

                    if (!isset($filters[$attributeCode])) {
                        $filters[$attributeCode] = $value;
                        unset($filters[$attributeCode]['swatch']);

                        if (!is_array($value['value'])) {
                            $filters[$attributeCode]['value'] = [$value['value']];
                        }

                        if (!is_array($value['key'])) {
                            $filters[$attributeCode]['key'] = [$value['key']];
                        }

                        continue;
                    }

                    if (is_array($value['value'])) {
                        array_merge($filters[$attributeCode]['value'], $value['value']);
                        array_merge($filters[$attributeCode]['key'], $value['key']);
                        continue;
                    }

                    $filters[$attributeCode]['value'][] = $value['value'];
                    $filters[$attributeCode]['key'][] = $value['key'];
                }
            }

            $parentFilters = $refiloProduct->getData(Product::FILTERS);
            array_merge($parentFilters, $filters);
            $refiloProduct->setData(Product::FILTERS, array_merge($parentFilters, $filters));
            $refiloProduct->setVariants($variants);
        }

        return $refiloProduct;
    }

    /**
     * @param OptionInterface $option
     * @return array
     */
    protected function getOptions(OptionInterface $option)
    {
        $output = [];
        foreach ($option->getOptions() ?? [] as $optionValue) {
            $data = $this->productAttributeProvider->getSwatchByOptionId($optionValue['value_index']);
            $data['value_index'] = $optionValue['value_index'];
            $data['label'] = $optionValue['store_label'];
            $output[] = $data;
        }

        return $output;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getBundleChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $extensionAttributes = $magentoProduct->getExtensionAttributes();
        $options = [];

        $refiloProduct->setData('bundle_price_type', (int)$magentoProduct->getPriceType() == 0 ? 'dynamic' : 'fixed');
        foreach ($extensionAttributes->getBundleProductOptions() as $bundleProductOption) {
            $productLinks = $bundleProductOption->getData('product_links');

            if (empty($productLinks)) {
                continue;
            }

            $option = [
                'id'       => (int)$bundleProductOption->getData('option_id'),
                'title'    => $bundleProductOption->getData('title'),
                'required' => (int)$bundleProductOption->getData('required'),
                'type'     => $bundleProductOption->getData('type'),
                'position' => (int)$bundleProductOption->getData('position'),
                'childs'   => []
            ];

            foreach ($productLinks as $productLink) {
                $link_data = $productLink->getData();
                $link_data['product_id'] = $link_data['entity_id'];
                $link_data['product_sku'] = $link_data['sku'];

                unset($link_data['entity_id']);
                unset($link_data['sku']);
                unset($link_data['option_id']);

                $child = [];
                foreach ($link_data as $key => $value) {
                    $child[$key] = (is_numeric($value)) ? $value + 0 : $value;
                }

                $option['childs'][] = $child;
            }

            $options[] = $option;
        }

        $refiloProduct->setBundle($options);

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getGroupedChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $links = $magentoProduct->getTypeInstance()->getChildrenIds($magentoProduct->getId());

        if (empty($links)) {
            return $refiloProduct;
        }

        $refiloProduct->setChildren(array_values(array_shift($links)));
        return $refiloProduct;
    }
}
